


void TrainAndExtrapolate(string &in netFile, string &in lessonFile, uint trainTime, 
                        uint extrapolateCount, string &in resultLessonName)
{
    // Open net, delete all lessons except for #1, create an additional, empty lesson #2
    // import training data into lesson #1
    OpenNet(netFile);
    SetLessonCount(1);
    SetLessonCount(2);
    SelectLesson(1);
    LoadLesson(lessonFile);
    
    // Train and save the net
    TrainForExtrapolation(trainTime);
    SaveNet();
    
    // Start recording of output activation values into lesson #2
    SetRecordingType(RT_ACT);
    StartRecording(2);
    ShowLessonEditor(false);
    // Put net into a well defined state (time variant network!)
    ResetNet();
    
    // Disable view update during think to increase speed
    ViewSetting(UPDATE_THINK, false);
    
    // Let training lesson run one time through the net
    ThinkLesson();
    SleepExec();
   
    // Up to now the output reaction of the net on the input of the currently known time series 
    // has been recorded.
    // From now on the net will calculate future data points. The first future data point is based on the 
    // last known target output value of the lesson since this is the last known data point from the past
    // (i.e. the 'now' data point). The script will now apply this last known data point to the input of the 
    // net and perform one single think step. This gives us the first future (i.e. extrapolated) data point.
    // For all following future data points the script will read the output of the net, apply it to the input 
    // and perform a think step.
    // All results will still be recorded into the result lesson since recording is still active.
    if (extrapolateCount > 0)
    {
        // Select the last pattern in the lesson from which we need the output value in order to 
        // apply it to the input of the net.
        SelectPattern(GetLessonSize());
        double act;
        // Get the lesson output and apply it to the net's input.
        bool ok = GetPatternOutput(1, act);
        
        ok = ok && ApplyInputAct(1, act);
        
        // Perform the think step.
        if (ok)
        {
            ThinkSteps(1);
            SleepExec();
        }
        
        // The first future data point has been acquired. Now go for the next ones using 
        // the current output of the net as next input.
        for (uint i = 1; ok && (i < extrapolateCount); i++)
        {
            ok = GetOutputAct(1, act);
            ok = ok && ApplyInputAct(1, act);
            ThinkSteps(1);
            SleepExec();
        } 
       
        if (!ok)
        {
            MessageBox("Error!");
        }
    }
    else
    {
        Trace("No future data points to extrapolate!\r\n");
    }
    
    // Stop recording, select lesson #2 and export to csv.
    StopRecording();
    ShowLessonEditor(true);
    
    // Re-enable view update during think
    ViewSetting(UPDATE_THINK, true);
                      
    SelectLesson(2);
    ExportLessonRaw(resultLessonName);
}


// Train the net for extrapolation.
void TrainForExtrapolation(uint trainTime)
{
    RandomizeNet();
    StartTimer(trainTime * 1000);

	ETeachResult res = TR_OK;

	StartTeaching();

	// Teach as long as teacher signals OK and as long as timer is not expired.
	// Update net error values during teaching at regular intervals
	while ((res == TR_OK) && !IsTimerExpired() && IsTeaching())
	{
		Sleep(100);
		
        res = GetLastTeachResult();
	}

	if (IsTeaching())
	{
		// Stop teaching for sure
		StopTeaching();
	}
}
